bias.msm <- function(formula.msm, data, gn.AW, gn.AV, Qn.mat, M, name.Y, name.A, script.A, family.msm="gaussian", tol=1e-6, max.iter=100, max.k=10, detail=FALSE){

  n <- nrow(gn.AW)
  nA <- ncol(gn.AW)
  nM <- length(M)
  
  if ((nrow(Qn.mat) != n) || (ncol(Qn.mat) != nA)) stop("gn.mat and Qn.mat must have the same dimensions")

  ## The true parameter value under the estimated
  ## data-generating distribution
  msm.fit0 <- get.truth(formula.msm, family.msm, data, Qn.mat, name.Y, name.A, script.A)
  beta0 <- as.numeric(coef(msm.fit0))  
  X0 <- model.matrix(msm.fit0)
  np <- ncol(X0)
  m.deriv0 <- msm.deriv(X0, beta0, family=family.msm)
  Y0 <- msm.fit0$data[,name.Y]

  wt.mat <- gn.AV/gn.AW
  
  wt <- as.vector(wt.mat)

  tol <- rep(tol, np)
  bias <- matrix(0, np, nM)
 
  for (j in 1:nM){
    wt.M <- ifelse(wt > M[j], M[j], wt)
    frac.M <- matrix(rep(gn.AW*wt.M,np), ncol=np)

    beta <- beta0
    delta <- 2*tol
    iter <- 0
    k <- 0
    res <- get.phi(beta, X0, Y0, frac.M, n, family.msm)      
    phi <- res$phi
    while(any(abs(phi) > tol) && iter <= max.iter && k <=max.k){      
      iter <-  iter + 1
 
      delta <- res$delta

      if (detail) cat("j =", j, "iter =", iter, "k =", k, "phi =", phi, "beta =", beta, "delta =", delta, "\n")      
      
      ## line search
      phi.k <- 2*phi
      k <- 0 
      while (any(abs(phi.k) > abs(phi)) && k <= max.k){
        beta.k <- beta - delta/(2^k)
        res <- get.phi(beta.k, X0, Y0, frac.M, n, family.msm)
        phi.k <- res$phi
        k <- k + 1
      }
         
      ## update beta 
      beta <- beta.k
      phi <- phi.k
      
    }
    
    bias[,j] <- beta - beta0
  } 

  rownames(bias) <- names(coef(msm.fit0))
  colnames(bias) <- paste("M=", M, sep="")

  bias
}



