tIPTW.msm <- function(formula.msm, formula.AW, mse.wts, data, formula.YAW=NULL, formula.AV=NULL, family.msm="gaussian", family.AW="binomial", M=c(2,5,10,25,50,100,500, Inf), tol=1e-8, max.iter=10){

  np <- 1 + ncol(attr(terms(formula.msm),"factors"))
  nM <- length(M)
  n <- nrow(data)
  
  if (length(mse.wts) != np) stop("Argument mse.wts has wrong length.")
  if (any(mse.wts<0)) stop("All elements in mse.wts must be non-negative.")
  if (sum(mse.wts)<=0) stop("There must be at least one element in mse.wts that is greater than zero.")
  if (!family.msm %in% c("gaussian", "binomial")) stop(paste("Argument", family.msm, "for family.msm currently not supported."))
  if (family.AW != "binomial") stop(paste("Argument", family.AW, "for family.AW currently not supported."))  
  if (is.null(formula.YAW) & family.AW != "binomial") stop("If A is not binary, a model for E[Y|A,W] must be supplied.")
  if (any(M<=0)) stop("All candidate truncation levels must greater than zero.")

  family.YAW <- family.msm
  family.AV <- family.AW
  name.A <- as.character(formula.AW)[2]
  name.Y <- as.character(formula.msm)[2]
  script.A <- seq(min(data[,name.A]), max(data[,name.A]))

  
  ## Fit g and Q and obtain matrix of fitted values
  gn.AW <- fit.g(formula.AW, family.AW, data)

  if (is.null(formula.YAW)){    
    Qn.mat <- fit.Q.ps(ps=gn.AW[,2], family.YAW, data, name.A, script.A, name.Y)
  }else{
    Qn.mat <- fit.Q(formula.YAW, family.YAW, data, name.A, script.A)
  }

  
  if (!is.null(formula.AV)){
    gn.AV <- fit.g(formula.AV, family.AV, data)
  }else{
    gn.AV <- matrix(1, nrow=nrow(gn.AW), ncol=ncol(gn.AW))
  }

  wt.mat <- gn.AV/gn.AW

  
  ## point estimates for different truncation levels
  coef.M <- matrix(0, np, nM)
  for (m in 1:nM){
    msm.fit <- fit.msm(formula.msm, formula.AW, M[m], data, formula.AV=formula.AV, family.msm=family.msm, family.AW=family.AW)
    coef.M[,m] <- coef(msm.fit)
  } 
  rownames(coef.M) <- names(coef(msm.fit))
  colnames(coef.M) <- paste("M=", M, sep="")
  
  ## Estimate variance  
  var <- var.msm(formula.msm, gn.AW, gn.AV, coef.M, M, data, name.A, name.Y, script.A, family.msm)
  
  ## Estimate bias 
  bias <- bias.msm(formula.msm, data, gn.AW, gn.AV, Qn.mat, M, name.Y, name.A, script.A, family.msm, tol=tol, max.iter=max.iter)

  
  ## Minimize weighted mean squared error to find optimal M
  mse <- var + bias^2   
  crit <- apply(mse*mse.wts, 2, sum)

  pick <- (1:length(M))[crit == min(crit)][1]
  M.opt <- M[pick]
  est.opt <- coef.M[,pick]

  list(est=est.opt, M=M.opt, est.M=coef.M, crit=crit, var=var, bias=bias, mse=mse)
}
