var.msm <- function(formula.msm, gn.AW, gn.AV, limits, M, data, name.A, name.Y, script.A, family.msm){

  n <- nrow(gn.AW)
  nA <- ncol(gn.AW)
  np <- 1 + ncol(attr(terms(formula.msm),"factors"))  
  nM <- length(M)
  
  A <- data[,name.A]
  Y <- data[,name.Y]
  
  ## make sure A is coded as 1,2,...
  if (min(A) != 1) A <- A - min(A) + 1

  ## get model matrix X
  msm.fit <- glm(formula.msm, family=family.msm, data=data)
  X <- model.matrix(msm.fit)

  
  ## model matrices for different values of A
  da <- data
  Xa <- array(0, dim=c(dim(X), nA))
  for (k in 1:nA){
    da[,name.A] <- script.A[k]
    Xa[,,k] <- model.matrix(formula.msm, data=da)
  }

  wt.mat <- gn.AV/gn.AW
  
  v.mat <- matrix(0, nrow=np,ncol=nM)
  for (j in 1:nM){
    lim <- limits[,j]
    
    wt.mat.M <- ifelse(wt.mat > M[j], M[j], wt.mat)
    wt.M <- wt.mat.M[cbind(1:n, A)]
    
    ## finding the standardization constant c using integration over A
    c <- matrix(0, np, np) 

    for (k in 1:nA){
      m.deriv.a <- msm.deriv(Xa[,,k], lim, family=family.msm)      
      h.AV <- matrix(1, nrow=nrow(gn.AW), ncol=ncol(gn.AW))
      if (family.msm == "binomial"){
        fitted <- msm.fitted(Xa[,,k], lim, family=family.msm)
        h.AV <- h.AV/matrix(rep(fitted*(1-fitted), np), ncol=np)
      }
      c <- c + t(m.deriv.a)%*%(m.deriv.a*matrix(rep(gn.AW[,k]*wt.mat.M[,k],np), ncol=np)*h.AV)/n
    }
    c.inv <- solve(c)
    

    
    ## finding the empirical variance of the IPTW influence curve
    fitted <- msm.fitted(X, lim, family=family.msm)
    resid <- Y - fitted
    m.deriv <- msm.deriv(X, lim, family=family.msm)

    ## for logistic models, weight by 1/variance
    h.AV <- matrix(1, nrow=nrow(gn.AW), ncol=ncol(gn.AW))
    if (family.msm == "binomial") h.AV <- h.AV / matrix(rep(fitted*(1-fitted), np), ncol=np)

    
    D.iptw <- matrix(rep(wt.M,np), ncol=np) * h.AV * m.deriv *  matrix(rep(resid,np), ncol=np)
    v.mat[,j] <- diag(c.inv%*%(var(D.iptw)/n)%*%c.inv) 
  }
  
  rownames(v.mat) <- names(coef(msm.fit))
  colnames(v.mat) <- paste("M=", M, sep="")

  
  v.mat


}
