\name{tIPTW.msm}
\alias{tIPTW.msm}
\title{Data-adaptively truncated ITPW estimator for MSMs}
\description{
  This function implements an IPTW estimator for marginal structural
  models that data-adaptively selects a truncation level for the
  treatment mechanism with the goal of minimizing the mean squared error
  of the estimator.
}
\usage{
  tIPTW.msm(formula.msm, formula.AW, mse.wts, data, formula.YAW = NULL,
    formula.AV = NULL, family.msm = "gaussian", family.AW = "binomial",
    M = c(2,5,10,25,50,100,500,Inf), tol = 1e-08, max.iter = 10) 
}
\arguments{
  \item{formula.msm}{A \code{formula} defining the marginal structural
  model}
  \item{formula.AW}{A \code{formula} defining the model for the
    treatment mechanism P(A|W).}
  \item{mse.wts}{A \code{vector} of length equal to the number of
    parameters in the marginal structural model that defines what weight
    to give to the mean squared error of these parameters in order to
    obtain a one-dimensional mean-squared-error criterion to minimize.}
  \item{data}{A \code{data.frame} containing the data.}
  \item{formula.YAW}{A \code{formula} defining the model for
    E[Y|A,W]. If A is binary, this can be set to NULL, in which the
    model will be based on a categorical version of the propensity score.}
  \item{formula.AV}{A \code{formula} defining the model for the
    numerator P(A|V) of the stabilized weights.}
  \item{family.msm}{The \code{family} of the marginal structural
    model. Currently only the gaussian and binomial families are supported.}
  \item{family.AW}{The \code{family} of the treatment
    mechanism. Currently only the binomial family is supported.}
  \item{M}{A \code{vector} containing the candidate truncation levels.}
  \item{tol}{The tolerance for the iterative bias calculation. Currently
  not supported.}
  \item{max.iter}{The maximum number of iterations allowed for the
    iterative bias calculation. Currently not supported.}
}
\details{
}
\value{
  Returns a list with the following components:
  \item{est}{The estimated coefficients for the marginal structral model.}
  \item{M}{The selected truncation level.}
  \item{est.M}{The estimated coefficients for the marginal structural
    model for each candidate truncation level.}
  \item{crit}{The value of the criterion function minimized by the
    data-adaptive selector for each candidate truncation level.}
  \item{var}{The estimated variance of the coefficients estimates for
    each candidate truncation level.} 
  \item{bias}{The estimated bias of the coefficients estimates for
    each candidate truncation level.} 
  \item{mse}{The estimated mean squared error of the coefficients
    estimates for each candidate truncation level.} 
}
\references{}
\author{Oliver Bembom}
\note{
}
\seealso{}
\examples{

formula.msm <- formula("Y ~ A")
formula.AW <- formula("A ~ W1 + W2")

## select truncation level to minimize the mean squared error
## for estimating the coefficient in front of A in the MSM
mse.wts <- c(0, 1)

## continuous outcome
data(linMSM)
res1 <- tIPTW.msm(formula.msm, formula.AW, mse.wts, data=linMSM)

## binary outcome
data(logMSM)
res2 <- tIPTW.msm(formula.msm, formula.AW, mse.wts, data=logMSM, family.msm="binomial")

## stabilized weights
formula.AV <- formula("A~1")
res3 <- tIPTW.msm(formula.msm, formula.AW, mse.wts, data=linMSM, formula.AV=formula.AV)



}
\keyword{misc}
